/**
 * Nicula Iulia Alexandra
 * grupa 334 CB
 * Proiect PM 2017 -- CUBUL
 *------------------------------------------
 * Implementare jocuri de lumini pentru cubul de LED-uri
 */

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
#include "utils.h"

#define NIVEL1 0
#define NIVEL2 1
#define NIVEL3 2
#define NIVEL4 3

#define DELAY_MS 500
/* efectul care trebuie redat */
static int efect = 0;

/* functii */
void coloana(void);
void sus_jos(void);
void led_random(void);
void drum_sinuos(void);
void aprindere_colturi(void);
void aprinde_fata_laterala(void);
void convergent(void);
void aprindere_diagonala(void);
void complet(void);
void coloana_centru(void);
void aprindere_jumatate(void);
void sarpe(void);

static bool apasat = false;

/**
 * Intrerupere de comparatie cu OCR1A
 * Se va verfica starea butonului din 10 in 10 ms
 */
ISR(TIMER1_COMPA_vect)
{
    if((PIND & (1 << PD6)) == 0 && !apasat)
    {
        efect = (efect + 1) % 13;
        PORTD ^= (1 << PD7);
        apasat = true;
    }
    else if((PIND & (1 << PD6)) != 0)
        apasat = false;
}


unsigned int A[8] = {PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7};
unsigned int B[8] = {PB0, PB1, PB2, PB3, PB4, PB5, PB6, PB7};

/* jocurile de lumini ce vor fi redate */
functie_efect joc_lumini[13] = {
                                    coloana, 
                                    sus_jos, 
                                    led_random,
                                    drum_sinuos,
                                    aprindere_colturi,
                                    aprinde_fata_laterala,
                                    aprindere_diagonala,
                                    complet,
                                    coloana_centru,
                                    resetare,
                                    aprindere_jumatate,
                                    sarpe,
                                    convergent
                                };
                                
/** 
 * Aprindere completa a cate unei sectiuni longitudinale
 */
void coloana(void) {
    static int first = 0;
    static int second = 4;  
    PORTA |= (1 << A[first]);
    PORTA |= (1 << A[second]);
    PORTB |= (1 << B[first]);
    PORTB |= (1 << B[second]);

    _delay_ms(100);

    PORTA &= ~(1 << A[first]);
    PORTA &= ~(1 << A[second]);
    PORTB &= ~(1 << B[first]);
    PORTB &= ~(1 << B[second]);     
    
    first = (first + 1) % 4;
    second = (second + 1) % 4 + 4;
        

}

/**
 * Aprindere completa a cate unui nivel, de sus in jos si invers
 */
void sus_jos(void) {
    static int i = 0;
    static int sign = 1;
    
    selecteaza_nivel(i);
    _delay_ms(200);
    stinge_nivel(i);
    
    if (i == 3 && sign == 1) {
        sign = -1;
        _delay_ms(200);
    }
    
    if (i == 0 && sign == -1)
        sign = 1;
    i += sign;
}

/**
 * Aprindere in mod aleatoriu a unui led
 */
void led_random() {
    selecteaza_nivel(rand() % 4);

    if (rand() % 2)
        PORTA |= (1 << A[rand() % 8]);
    else
        PORTB |= (1 << B[rand() % 8]);
    _delay_ms(100);
    
    resetare();
}

/**
 * Animatie de aprindere a cate unei coloane de led-uri, urmand un traseu sinuos
 */

void drum_sinuos(void) {
    static int position = 0;
    static int sign = 1;
    static bool okA = true;
    static bool okB = false;

    if (okA == true) {
        PORTA |= (1 << A[position]);
        _delay_ms(300);
        PORTA &= ~(1 << A[position]);
        
        if (position == 3) {
            position = 7;
            sign = -1;
            return;
        }
        
        if (position == 4) {
            okA = false;
            okB = true;
            position = 0;
            sign = 1;
            return;
        }
        position += sign;
    } else {
        PORTB |= (1 << B[position]);
        _delay_ms(DELAY_MS);
        PORTB &= ~(1 << B[position]);               
        
        if (position == 3) {
            position = 7;
            sign = -1;
            return;
        }
        
        if (position == 4) {
            okB = false;
            okA = true;
            position = 0;
            sign = 1;
            return;
        }
        position += sign;
    }
}

/**
 * Aprinderea ledurilor de pe una dintre fetele laterale
 */
 
static int fata = 0;
void aprinde_fata_laterala(void) {
    switch (fata) {
    case 0:
        PORTA |= (1 << A[0]) | (1 << A[4]);
        PORTB |= (1 << B[0]) | (1 << B[4]);
        break;
    case 1:
        PORTA |= (15 << A[0]);
        break;
    case 2:
        PORTA |= (1 << A[3]) | (1 << A[7]);
        PORTB |= (1 << B[3]) | (1 << B[7]);
        break;
    case 3:
        PORTB |= (15 << B[4]);
    default:
        break;
    }
}

/**
 * Efect plan superior: stinge cate o coloana
 */
void convergent(void) {
    static int stare = 0;
    static int nivel = NIVEL4;
    
    selecteaza_nivel(nivel);
    
    switch (stare) {
    case 0:
        PORTA |= (15 << A[0]) | (1 << A[4]) | (1 << A[7]);
        PORTB |= (15 << B[4]) | (1 << B[0]) | (1 << B[3]);
        break;
    case 1:
        PORTA |= (3 << A[5]);
        PORTB |= (3 << B[1]);
        break;
    case 2:
        resetare();
        nivel = (nivel == NIVEL1) ? NIVEL4 : (nivel - 1);
        break;
    default:
        break;
    }
    
    stare = (stare + 1) % 3;
}

/**
 * Aprinderea ledurilor de pe diagonala principala
 */
void aprindere_diagonala(void) {
    static int nivel;
    
    selecteaza_nivel(nivel);
    switch (nivel) {
    case 0:
        PORTA |= (15 << A[0]);
        _delay_ms(50);
        PORTA &= ~(15 << A[0]);
        break;
    case 1:
        PORTA |= (15 << A[4]);
        _delay_ms(50);
        PORTA &= ~(15 << A[4]);
        break;
    case 2:
        PORTB |= (15 << B[0]);
        _delay_ms(50);
        PORTB &= ~(15 << B[0]); 
        break;
    case 3:
        PORTB |= (15 << B[4]);
        _delay_ms(50);
        PORTB &= ~(15 << B[4]); 
        break;
    default:
        break;
    }
    
    stinge_nivel(nivel);
    nivel = (nivel + 1) % 4;
}

/**
 * Aprindere integrala a cubului de leduri
 */
void complet(void) {
    static int nivel = NIVEL1;
        
    PORTA = 0xFF;
    PORTB = 0xFF;

    selecteaza_nivel(nivel);
    _delay_ms(2);
    stinge_nivel(nivel);
    nivel = (nivel == 0) ? 3 : (nivel-1);
}


/**
 * Jumatate din led aprinsa *
 *                          **
 *                          ***
 *                          ****
 */
void aprindere_jumatate(void) {
    static int nivel = NIVEL1;
    resetare();
    selecteaza_nivel(nivel);
    
    switch (nivel) {
    case 0:
        PORTA = 0xFF;
        PORTA &= ~(15 << A[0]);
        PORTB = 0xFF;
        break;
    case 1:
        PORTB = 0xFF;
        break;
    case 2:
        PORTB |= (15 << B[4]);
        break;
    default:
        break;
    }
    
    _delay_ms(2);
    nivel = (nivel + 1) % 3;

}

/**
 * Alternarea coloanelor din centrul cubului
 */
void coloana_centru(void) {
    static int coloana = 0;
    unsigned int centru[4] = {A[5], A[6], B[2], B[1]};
    
    if (coloana == 0 || coloana == 1) {
        PORTA |= (1 << centru[coloana]);
        _delay_ms(200);
        PORTA &= ~(1 << centru[coloana]);
    } 
    else {
        PORTB |= (1 << centru[coloana]);
        _delay_ms(200);
        PORTB &= ~(1 << centru[coloana]);
    }
    
    coloana = (coloana + 1) % 4;

}

/**
 * Aprinderea led-urilor pentru crearea unei animatii de sarpe care
 * se deplaseaza
 */
void sarpe(void) {
    static int stare = 0;
    static int sign = 1;
    
    switch (stare) {
    case 0:
        PORTA |= (15 << A[0]);
        break;
    case 1:
        PORTA |= (14 << A[0]);
        PORTA |= (1 << A[7]);
        break;
    case 2:
        PORTA |= (3 << A[2]);
        PORTA |= (3 << A[6]);
        break;
    case 3:
        PORTA |= (1 << A[3]);
        PORTA |= (7 << A[5]);
        break;
    case 4:
        PORTA |= (15 << A[4]);
        break;
    case 5:
        PORTA |= (7 << A[4]);
        PORTB |= (1 << B[0]);
        break;
    case 6:
        PORTA |= (3 << A[4]);
        PORTB |= (3 << B[0]);
        break;
    case 7:
        PORTA |= (1 << A[4]);
        PORTB |= (7 << B[0]);
        break;
    case 8:
        PORTB |= (15 << B[0]);
        break;
    case 9:
        PORTB |= (7 << B[1]);
        PORTB |= (1 << B[7]);
        break;
    case 10:
        PORTB |= (3 << B[2]);
        PORTB |= (3 << B[6]);
        break;
    case 11:
        PORTB |= (7 << B[5]);
        PORTB |= (1 << B[3]);
        break;
    case 12:
        PORTB |= (15 << B[4]);
        break;
    default:
        break;  
    }
    
    if (stare == 12) {
        sign = -1;
        _delay_ms(DELAY_MS);
    }
    
    
    if (stare == 0) {
        sign = 1;
        _delay_ms(DELAY_MS);
    }
    
    stare += sign;

}

void aprindere_colturi(void) {
    selecteaza_nivel(NIVEL1);
    selecteaza_nivel(NIVEL4);
    
    PORTA |= (1 << A[0]) | (1 << A[3]);
    PORTB |= (1 << B[4]) | (1 << B[7]);
}

int main(void) {

    sei();

    initializare();

    while (1) {
        switch (efect) {
        case 0:
        /* aprindere longitudinal */
            resetare();
            select_toate_niv();         
            break;
        case 1:
        /* aprindere transversal*/
            PORTA = 0xFF;
            PORTB = 0xFF;       
            break;
        case 2:
        /* aprindere led random */
            resetare();
            break;
        case 3:
        /* drum sinuos */
            resetare();
            select_toate_niv();
            break;
        case 4:
        /* aprinderea ledurilor din colturile cubului*/
            resetare();
            joc_lumini[efect]();
            break;
        case 5:
        /* aprindere fata laterala */
            resetare();
            select_toate_niv();
            fata = (fata == 3) ? 0 : (fata + 1);
            joc_lumini[efect]();
            _delay_ms(200);
            continue;
        case 6:
        /* aprindere_diagonala */
            resetare();
            break;
        case 7:
        /* complet */
            resetare();
            break;
        case 8:
        /* coloana centru */
            resetare();
            select_toate_niv();
            break;
        case 9:
        /* stins */
            break;
        case 10:
        /* aprindere jumatate */
            resetare();
            break;
        case 11:
        /* sarpe */
            resetare();
            selecteaza_nivel(NIVEL4);
            joc_lumini[efect]();
            _delay_ms(DELAY_MS);
            continue;
        case 12:
        /* convergent */
            _delay_ms(DELAY_MS);
            resetare();
            break;
        default:
            resetare();
        }
        joc_lumini[efect]();
    }
    return 0;
}
